#include <iostream>
#include "farm.h"
#include "pipe.h"
#include "initial.h"
#include "final.h"
#include "functors.h"
#include "darray.h"

static int result = 0;
static int current = 0;

int id(int a)
{
  return a;
}

struct Add1 : msl::FarmFunctor<int, int>
{
  msl::LArray<int> la;

  Add1(msl::DArray<int>& da) : la(da, msl::Distribution::DIST)
  {
    this->addArgument(&la);
  }

	MSL_USERFUNC
	int operator()(int a) const
	{
	  printf("size: %d\n", la.getSize());
	  for (int i = 0; i < la.getSize(); i++) {
	    printf("%d, ", la[i]);
	  }
	  printf("\n");
		return a + 1;// + la[0];
	}
};

struct Init
{
	int n = 2;

	int* operator()()
	{
		if (current < n) {
			return new int(current++);
		} else
			return 0;
	}
};

struct Fin
{
	void operator()(int n)
	{
		result += n;
	}
};

int main(int argc, char** argv)
{
	bool debugCommunication = 0;
	msl::initSkeletons(argc, argv, debugCommunication);

	int nWorkers = 1;
	int nRuns = 1;
	int nThreads = 4;
	if (argc < 5) {
		if (msl::isRootProcess()) {
			std::cout << "Usage: " << argv[0] << " nWorkers nThreads nRuns cpuProcessing" << std::endl;
		}
	} else {
		nWorkers = atoi(argv[1]);
		nThreads = atoi(argv[2]);
		nRuns = atoi(argv[3]);
	}

	msl::setNumGpus(2);
	msl::setTaskGroupSize(1);
	msl::setThreadsPerBlock(256);
	msl::setNumThreads(nThreads);
	msl::setNumRuns(nRuns);
	msl::setNumConcurrentKernels(4);

	msl::DArray<int> da(8, id, msl::Distribution::COPY);

	Init init;
	Add1 add1(da);
	Fin fin;

	msl::Initial<int, Init> p1(init);
	msl::Farm<int, int, Add1> p3(add1, nWorkers, 0);
	msl::Final<int, Fin> p4(fin);
	msl::Pipe p5(p1, p3, p4);

	// start the calculation.
	for (int i = 0; i < msl::Muesli::num_runs; i++) {
		p5.start();
		MPI_Barrier (MPI_COMM_WORLD);
		current = 0;
	}

	if (msl::Muesli::proc_id == msl::Muesli::num_total_procs - 1)
		std::cout << "Result: " << result << std::endl;

	msl::terminateSkeletons();
}
